(function (global) {
    "use strict";

    global.ND = global.ND || {};
    global.ND.Portal = global.ND.Portal || {};
    global.ND.Portal.Documents = (function() {

		var currentDocumentsPage = 1;

        var createDocumentsRow = function(result) {
            $("#documents-count").show();
            $("#documents-count-value").text(result.TotalRecordCount);
            return `
                <tr>
                    <td data-th="Document">
                        <a>${result.DocumentTitle}</a>
                    </td>
                    <td data-th="Actions">
                        <a href="javascript:void(0)" onclick="openApplication('${result.DocumentTitle}')"><span class="glyphicon glyphicon-trash"> </span></a>
                        
                    </td>
                </tr>`;
        };
        
		var _readFileAsync = async (file) => {
			let result_base64 = await new Promise((resolve) => {
				let fileReader = new FileReader();
				fileReader.onload = (e) => resolve(fileReader.result);
				fileReader.readAsDataURL(file);
			});
			return result_base64;
		};
		
		var handleFileSave = async (applicationId) => {		
		    //event.preventDefault();
			const fileInput = $("#AttachFile")[0]?.files;
			if (fileInput?.length === 0) {
				return;
			}
			
			const file = fileInput[0];
            const fileName = file.name;
            const mimeType = file.type;
			const base64File = await _readFileAsync(file);
			const base64Content = base64File.split(',')[1];
			const note = {
				"objectid_eyland_application@odata.bind": `/eyland_applications(${applicationId})`,
				"objecttypecode": "eyland_application",
				"subject": fileName,
				"filename": fileName,
				"documentbody": base64Content,
				"mimetype": mimeType
			};
			return appAjax('Saving record...', {
				type: "POST",
				url: "/_api/annotations",
				contentType:"application/json",
				data: JSON.stringify(note),
				success: function(res,status,xhr) {      
					return Promise.resolve();
				},
				error: function(xhr, status, error){
					var errorMessage = xhr.status + ': ' + xhr.statusText;
					console.log(errorMessage);
				}
			});	
		}

		var _documentId, _documentTitle;
		var _downloadDocumentFromApplication = () => {
			const documentBody = documentBodies[_documentId];
			if (!documentBody) {
				console.error("Document body not found for ID:", _documentId);
				return;
			}
	
			const binaryString = window.atob(documentBody);
			const binaryLen = binaryString.length;
			const bytes = new Uint8Array(binaryLen);
	
			for (let i = 0; i < binaryLen; i++) {
				bytes[i] = binaryString.charCodeAt(i);
			}
	
			const blob = new Blob([bytes], { type: "application/octet-stream" });
			const link = document.createElement("a");
			link.href = URL.createObjectURL(blob);
			link.download = _documentTitle;
			document.body.appendChild(link);
			link.click();
	
			document.body.removeChild(link);
		}

		var downloadPortalDocument = async (noteId, noteTitle, createDocumentsPostback) => {
			let baseUrl = fileNetBaseUrl;
			
			var isSuccess = await ND.Portal.FileNet.downloadDocumentV2(
				noteId,
				'application/pdf',
				baseUrl,
				'documents'
			);
	
			var endpoint;
			if (createDocumentsPostback.name === "createAttachmentItem") {
				endpoint = "ND Service: Get Message Center Documents";
			} else {
				endpoint = "ND Service: Get Application ROW Documents";
			}
			if (!isSuccess) {
				_documentId = noteId;
				_documentTitle = noteTitle;
				ND.Portal.Tables.handlePageToggle(1, 1, "documents", createDocumentsPostback, endpoint, null, null, _downloadDocumentFromApplication);
			}
		};

		var loadDocumentTable = async (baseUrl, entityName, createDocumentsPostback, applicationStatus) => {
			$('#documentsLoading').show();
			const urlParams = new URLSearchParams(window.location.search);
			var entityId;
			if (entityName === "ndland_portalmessage") {
				entityId = $("#nd-view-message-modal").attr("data-related-id");
			} else {
				entityId = urlParams.get('id');
			}
			let requestUrl = `${baseUrl}/api/documents/entity?entityName=${entityName}&entityId=${entityId}`;
			ND.Portal.API.appAjax('Downloading document...', {
				type: "GET",
				url: requestUrl,
				contentType: "application/json",
				success: function (results) {
					if (results.length > 0) {
						$("#documentsTable .empty-message").hide();
						$("fieldset[aria-label='Documents'] label").remove();
						$("#documents-count-value").text(results.length);
						results.forEach(result => {
							const dateObject = new Date(result.Date);
							const year = dateObject.getUTCFullYear();
							const month = String(dateObject.getUTCMonth() + 1).padStart(2, '0');
							const day = String(dateObject.getUTCDate()).padStart(2, '0');
							const formattedDate = `${month}/${day}/${year}`;
							
							var html = createDocumentsPostback({ 
								"FileName": result.DocumentTitle, 
								"DocumentTitle": result.DocumentTitle, 
								"DocumentId": result.DocumentId, 
								"Subject": result.SubjectTitle, 
								"Type": result.SurfaceDocumentType,
								"DateUploaded": formattedDate
							 });
							 if (entityName === "ndland_portalmessage" && results.length === 1) {
								var tempDiv = document.createElement('div');
								tempDiv.innerHTML = html.trim();
								var content = tempDiv.firstChild.innerHTML;
								$("#attachments-list-single").after(content);
								$("#attachments-list-multiple").hide();
							 } else if (entityName === "ndland_portalmessage" && results.length > 1) {
								$('#attachments-list-multiple').append(html);
								$("#attachments-list-single").hide();
							 } else {
								$("#documentsTable tbody").append(html);
							 }
						});
					} else {
						if (applicationStatus === "Approved" || applicationStatus === "Withdrawn") {
							$("fieldset[aria-label='Documents'] label").text("\u2014");
							$("fieldset[aria-label='Documents'] .table-container").remove();
						}
						console.log('No document found');
					}
                    
                    ND.Portal.Tables.updateSubgridCount('documents', results, results.length);
					$('#documentsLoading').hide();
                    $("#attachments-list-single").hide();
                    $("#attachments-list-multiple").hide();
					$("fieldset[aria-label='Documents'] .table-container").addClass("mt-0");
				},
				failure: function (res) {
					console.log('Error retrieiving document');
					$("#documentsLoading").hide();
				}
			});
		};
        
		var storeFileData = async (fileArray) => {
			const fileDataPromises = fileArray.map(async (file) => {
				try {
					console.log(`Reading file: ${file.name}`);
					const base64File = await _readFileAsync(file);
					const base64Content = base64File.split(',')[1];
					console.log(`Finished reading file: ${file.name}`);
					return {
						name: file.name,
						size: file.size,
						type: file.type,
						base64Content: base64Content
					};
				} catch (error) {
					console.error(`Error reading file: ${file.name}`, error);
					return null;
				}
			});
			try {
				const fileData = await Promise.all(fileDataPromises);
				console.log('All files have been read:', fileData);
				$("#file-data-hdn").val(JSON.stringify(fileData));
				return fileData;
			} catch (error) {
				console.error('An error occurred while processing the files:', error);
				throw error;
			}
		};

        return {
            handleFileSave: handleFileSave,
			downloadPortalDocument: downloadPortalDocument,
			loadDocumentTable: loadDocumentTable,
			storeFileData: storeFileData
        }
    })();

})(this);